﻿using UnityEditor;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;
using Utilities.Edit;

namespace Commons.UI
{
    public class UIWindowEditor : EditorWindow
    {
        Editor gameObjectEditor;

        #region 공통 변수

        private static EditorWindow window;
        private int toolBarIdx = 0;

        Vector2 scrollPosition = Vector2.zero;

        #endregion

        #region 이미지 변수

        private GameObject noneGameObject;
        private Sprite noneImage;

        private string noneName = "";
        private Transform noneParent;
        private string noneImageName = "";

        #endregion

        #region 버튼 변수

        // Button UI
        // Common
        private int buttonType;
        private Transform position;
        private string itemName = "";

        private string texturePathName = "";
        private Sprite sprite;

        private int animationType;
        private AnimationClip pressAnimation;
        private AnimationClip releaseAnimation;
        private AnimationClip clickAnimation;

        // image + text
        private Font text;
        private int textSize;
        private Color textColor;
        private string textString = "";
        private string textPathName = "";
        private TextAnchor textAnchor;

        // image + image
        private string texturePathName2 = "";
        private Sprite sprite2;

        // Add Button Script
        private string buttonScriptName = "";
        private string buttonScriptNamespaceName = "";
        private string buttonScriptCreatePath = "";
        private string[] tempScript = new string[]
        {
        "using Commons.UI;",
        "using UnityEngine.EventSystems;",
        " ",
        "namespace Menu.UI",
        "{",
        "    public class TempButtonScript : ButtonUI, IPointerClickHandler,",
        "        IPointerDownHandler,",
        "        IPointerUpHandler,",
        "        IPointerEnterHandler,",
        "        IPointerExitHandler,",
        "        ISelectHandler",
        "    {",
        "        public void OnPointerEnter(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerExit(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerClick(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerDown(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerUp(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnSelect(BaseEventData evd)",
        "        {",
        "",
        "        }",
        "    }",
        "}",
        };


        private string[] tempScript2 = new string[]{
        "using Commons.UI;",
        "using UnityEngine.EventSystems;",
        " ",
        "namespace Menu.UI",
        "{",
        "    public class TempButtonScript : ButtonUI, IPointerClickHandler,",
        "        IPointerDownHandler,",
        "        IPointerUpHandler,",
        "        IPointerEnterHandler,",
        "        IPointerExitHandler,",
        "        ISelectHandler",
        "    {",
        "        public void OnPointerEnter(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerExit(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerClick(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerDown(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnPointerUp(PointerEventData evd)",
        "        {",
        "",
        "        }",
        "        public void OnSelect(BaseEventData evd)",
        "        {",
        "",
        "        }",
        "    }",
        "}",
        };

        #endregion

        #region 팝업 변수

        private string popupName = "";
        private Transform popupPosition;

        private string popupBGName = "";
        private Color popupBGColor;
        private AnimationClip popupStartBGAnimation;
        private AnimationClip popupCloseBGAnimation;

        private string popupPanelName = "";
        private Sprite popupPanelBGImg;
        private AnimationClip popupStartPanelAnimation;
        private AnimationClip popupClosePanelAnimation;

        private ButtonUI popupCloseButton;
        private Vector2 popupCloseButtonCreateScrollPos;
        private GameObject popupButtonCheck;
        private bool popupButtonFold;


        // Add Button Script
        private string popupScriptName = "";
        private string popupScriptNamespaceName = "";
        private string popupScriptCreatePath = "";
        private string[] tempPopupScript = new string[]{
        "using Commons.UI;",
        " ",
        "namespace Menu.UI",
        "{",
        "    public class TempPopupScript : PopupUI",
        "    {",
        "        public override void AdditionalHideFunction(params object[] datas)",
        "        {",
        "        ",
        "        }",
        "        ",
        "        public override void AdditionalShowFunction(params object[] datas)",
        "        {",
        "         ",
        "        }",
        "    }",
        "}",
        };

        private string[] tempPopupScript2 = new string[]{
        "using Commons.UI;",
        " ",
        "namespace Menu.UI",
        "{",
        "    public class TempPopupScript : PopupUI",
        "    {",
        "        public override void AdditionalHideFunction(params object[] datas)",
        "        {",
        "        ",
        "        }",
        "        ",
        "        public override void AdditionalShowFunction(params object[] datas)",
        "        {",
        "         ",
        "        }",
        "    }",
        "}",
        };

        #endregion

        #region 스크롤 변수

        #endregion 

        void OnGUI()
        {
            ToolBar( new string[3] { "UI Image", "UI Button", "UI Popup" } );

            scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition);

            if (toolBarIdx == 0)
                ImageSection();
            else if (toolBarIdx == 1)
                ButtonSection();
            else if (toolBarIdx == 2)
                PopupSection();

            EditorGUILayout.EndScrollView();
        }

        #region Common Functions..

        private void ToolBar(string[] str)
        {
            toolBarIdx = GUILayout.Toolbar(toolBarIdx, str);
        }
        
        private void BlankSpace()
        {
            EditorGUILayout.Space();
            EditorGUILayout.Space();
            EditorGUILayout.Space();
        }

        #endregion

        #region Image Functions..

        private void ImageSection()
        {
            EditorGUILayout.HelpBox("이미지를 설정하여 사이즈에 맞게 자동으로 만듭니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion
            
            noneImage = (Sprite)EditorGUILayout.ObjectField("이미지", noneImage, typeof(Sprite), true);
            noneParent = (Transform)EditorGUILayout.ObjectField("생성할 위치", noneParent, typeof(Transform), true);

            #region 개행
            BlankSpace();
            #endregion

            if (noneName.Equals(string.Empty)) noneName = "TempImage";
            noneName = EditorGUILayout.TextField("이미지 이름", noneName);
            if (noneImageName.Equals(string.Empty)) noneImageName = "TempImageChild";
            noneImageName = EditorGUILayout.TextField("이미지 자식 이름", noneImageName);

            #region 개행
            BlankSpace();
            #endregion

            if (GUILayout.Button("이미지 생성"))
            {
                GameObject obj = EditorExtention.CreateChildObject(noneParent, noneName);
                GameObject obj2 = EditorExtention.CreateChildObject(obj.transform, noneImageName, typeof(Image));

                obj2.GetComponent<Image>().sprite = noneImage;
                obj2.GetComponent<Image>().SetNativeSize();
            }
            //if (noneGameObject != null)
            //{
            //    if (gameObjectEditor == null)
            //        gameObjectEditor = Editor.CreateEditor(noneGameObject);

            //    gameObjectEditor.OnPreviewGUI(GUILayoutUtility.GetRect(500, 500), GUIStyle.none);
            //}
        }

        #endregion

        #region Button Fucntion..

        private void ButtonToolBar(string[] str)
        {
            buttonType = GUILayout.Toolbar(buttonType, str);
        }

        private void ButtonSection()
        {
            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("버튼에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            // 메인 버튼 부모의 이름
            if (itemName.Equals(string.Empty)) itemName = "TempButton";
            itemName = EditorGUILayout.TextField("버튼의 이름", itemName);
            position = EditorGUILayout.ObjectField("만들어질 경로", position, typeof(Transform)) as Transform;

            #region 개행
            BlankSpace();
            #endregion

            if (buttonScriptName.Equals(string.Empty)) buttonScriptName = "TempButtonScript";
            buttonScriptName = EditorGUILayout.TextField("버튼 스크립트의 이름", buttonScriptName);
            if (buttonScriptNamespaceName.Equals(string.Empty)) buttonScriptNamespaceName = "Menu.UI";
            buttonScriptNamespaceName = EditorGUILayout.TextField("버튼 스크립트 네임스페이스", buttonScriptNamespaceName);
            if (buttonScriptCreatePath.Equals(string.Empty)) buttonScriptCreatePath = "Menu/UI/Scripts/";
            buttonScriptCreatePath = EditorGUILayout.TextField("버튼 스크립트 생성 위치", buttonScriptCreatePath);

            if (GUILayout.Button("스크립트 생성"))
                CreateButtonScript();

            #region 개행
            BlankSpace();
            #endregion

            // 버튼 툴바
            ButtonToolBar(new string[4] { "Only Image", "Only Text", "Image + Text", "Image + Image" });

            #region 개행
            BlankSpace();
            #endregion

            // 버튼 타입별로 분리
            if (buttonType == 0) OnlyImageButton();
            else if (buttonType == 1) TextButton();
            else if (buttonType == 2) ImageTextButton();
            else ImageAndImageButton();
        }

        private void OnlyImageButton()
        {
            EditorGUILayout.HelpBox("이미지에 대해서 설정하는 부분입니다.", MessageType.Info);
            
            #region 개행
            BlankSpace();
            #endregion

            if (texturePathName == string.Empty)
                texturePathName = "Image";

            sprite = EditorGUILayout.ObjectField("만들 이미지 등록", sprite, typeof(Sprite)) as Sprite;

            texturePathName = EditorGUILayout.TextField("만들어질 자식의 이름", texturePathName);
            
            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("애니메이션에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            animationType = GUILayout.Toolbar(animationType, new string[2] { "Press & Release", "Click" });

            #region 개행
            BlankSpace();
            #endregion

            if (animationType == 0)
            {
                pressAnimation = EditorGUILayout.ObjectField("누를 시 애니메이션", pressAnimation, typeof(AnimationClip)) as AnimationClip;
                releaseAnimation = EditorGUILayout.ObjectField("땔 시 애니메이션", releaseAnimation, typeof(AnimationClip)) as AnimationClip;
            }

            else
                clickAnimation = EditorGUILayout.ObjectField("클릭 시 애니메이션", clickAnimation, typeof(AnimationClip)) as AnimationClip;
            
            #region 개행
            BlankSpace();
            #endregion

            if (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                GUI.enabled = false;

            if (GUILayout.Button("Create Button"))
            {
                GameObject obj = EditorExtention.CreateChildObject(position, itemName, typeof(BoxCollider), typeof(Button), typeof(EventTrigger));
                UnityEngineInternal.APIUpdaterRuntimeServices.AddComponent(obj, "Assets/Menu/UI/Scripts/Commons/Editor/UIWindowEditor.cs (208,17)", buttonScriptName);
                GameObject obj2 = EditorExtention.CreateChildObject(obj.transform, texturePathName, typeof(Image));

                if (sprite != null) { obj2.GetComponent<Image>().sprite = sprite; obj2.GetComponent<Image>().SetNativeSize(); obj.GetComponent<BoxCollider>().size = new Vector3(obj2.GetComponent<Image>().preferredWidth, obj2.GetComponent<Image>().preferredHeight, 0); }

                obj.GetComponent<Button>().transition = Selectable.Transition.None;

                if (pressAnimation != null || releaseAnimation != null || clickAnimation != null) EditorExtention.AddComponent(obj, typeof(Animation));
                if (pressAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonAnimation));
                    obj.GetComponent<Animation>().AddClip(pressAnimation, "PressAnimation");
                }
                if (releaseAnimation != null) obj.GetComponent<Animation>().AddClip(releaseAnimation, "ReleaseAnimation");
                if (clickAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonClickAnimation));
                    obj.GetComponent<Animation>().AddClip(clickAnimation, "ClickAnimation");
                }

                Selection.activeGameObject = obj;
                popupButtonCheck = obj;
            }


            if (GUI.enabled == false)
                GUI.enabled = true;
        }

        private void ImageAndImageButton()
        {
            EditorGUILayout.HelpBox("이미지에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            if (texturePathName == string.Empty)
                texturePathName = "Image";

            sprite = EditorGUILayout.ObjectField("만들 이미지 등록", sprite, typeof(Sprite)) as Sprite;

            texturePathName = EditorGUILayout.TextField("만들어질 자식의 이름", texturePathName);

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("두번째 이미지에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            if (texturePathName2 == string.Empty)
                texturePathName2 = "Image2";

            sprite2 = EditorGUILayout.ObjectField("만들 이미지 등록", sprite2, typeof(Sprite)) as Sprite;

            texturePathName2 = EditorGUILayout.TextField("만들어질 자식의 이름", texturePathName2);

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("애니메이션에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            animationType = GUILayout.Toolbar(animationType, new string[2] { "Press & Release", "Click" });

            #region 개행
            BlankSpace();
            #endregion

            if (animationType == 0)
            {
                pressAnimation = EditorGUILayout.ObjectField("누를 시 애니메이션", pressAnimation, typeof(AnimationClip)) as AnimationClip;
                releaseAnimation = EditorGUILayout.ObjectField("땔 시 애니메이션", releaseAnimation, typeof(AnimationClip)) as AnimationClip;
            }
            else
                clickAnimation = EditorGUILayout.ObjectField("클릭 시 애니메이션", clickAnimation, typeof(AnimationClip)) as AnimationClip;

            #region 개행
            BlankSpace();
            #endregion

            if (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                GUI.enabled = false;

            if (GUILayout.Button("Create Button"))
            {
                CreateButtonScript();

                while (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                {
                    // 에디터 업데이트
                    EditorApplication.update();

                    // 에셋 초기화 (최신으로)
                    AssetDatabase.Refresh();
                }

                GameObject obj = EditorExtention.CreateChildObject(position, itemName, typeof(BoxCollider), typeof(Button), typeof(EventTrigger));
                UnityEngineInternal.APIUpdaterRuntimeServices.AddComponent(obj, "Assets/Menu/UI/Scripts/Commons/Editor/UIWindowEditor.cs (208,17)", buttonScriptName);
                GameObject obj2 = EditorExtention.CreateChildObject(obj.transform, texturePathName, typeof(Image));
                GameObject obj3 = EditorExtention.CreateChildObject(obj.transform, texturePathName2, typeof(Image));

                if (sprite != null) { obj2.GetComponent<Image>().sprite = sprite; obj2.GetComponent<Image>().SetNativeSize(); obj.GetComponent<BoxCollider>().size = new Vector3(obj2.GetComponent<Image>().preferredWidth, obj2.GetComponent<Image>().preferredHeight, 0); }
                if (sprite2 != null) { obj3.GetComponent<Image>().sprite = sprite2; obj3.GetComponent<Image>().SetNativeSize(); obj.GetComponent<BoxCollider>().size = new Vector3(obj3.GetComponent<Image>().preferredWidth, obj3.GetComponent<Image>().preferredHeight, 0); }

                obj.GetComponent<Button>().transition = Selectable.Transition.None;

                if (pressAnimation != null || releaseAnimation != null || clickAnimation != null) EditorExtention.AddComponent(obj, typeof(Animation));
                if (pressAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonAnimation));
                    obj.GetComponent<Animation>().AddClip(pressAnimation, "PressAnimation");
                }
                if (releaseAnimation != null) obj.GetComponent<Animation>().AddClip(releaseAnimation, "ReleaseAnimation");
                if (clickAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonClickAnimation));
                    obj.GetComponent<Animation>().AddClip(clickAnimation, "ClickAnimation");
                }

                Selection.activeGameObject = obj;
                popupButtonCheck = obj;
            }
            if (GUI.enabled == false)
                GUI.enabled = true;
        }

        private void ImageTextButton()
        {
            EditorGUILayout.HelpBox("이미지에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            if (texturePathName == string.Empty)
                texturePathName = "Image";

            sprite = EditorGUILayout.ObjectField("만들 이미지 등록", sprite, typeof(Sprite)) as Sprite;

            texturePathName = EditorGUILayout.TextField("만들어질 자식의 이름", texturePathName);

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("폰트에 대해서 설정하는 부분입니다.", MessageType.Info);
            
            #region 개행
            BlankSpace();
            #endregion

            text = EditorGUILayout.ObjectField("폰트 등록", text, typeof(Font)) as Font;

            if (textString.Equals(string.Empty)) textString = "Text";
            textString = EditorGUILayout.TextField("폰트 텍스트 등록", textString);
            if (textPathName.Equals(string.Empty)) textPathName = "Text";
            textPathName = EditorGUILayout.TextField("만들어질 자식의 이름", textPathName);
            if (textSize <= 0) textSize = 14;
            textSize = EditorGUILayout.IntField("폰트 사이즈 등록", textSize);
            textAnchor = (TextAnchor)EditorGUILayout.EnumPopup("폰트 정렬 기준 등록", textAnchor);
            textColor = EditorGUILayout.ColorField("폰트 색상 등록", textColor);

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("애니메이션에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            animationType = GUILayout.Toolbar(animationType, new string[2] { "Press & Release", "Click" });

            #region 개행
            BlankSpace();
            #endregion

            if (animationType == 0)
            {
                pressAnimation = EditorGUILayout.ObjectField("누를 시 애니메이션", pressAnimation, typeof(AnimationClip)) as AnimationClip;
                releaseAnimation = EditorGUILayout.ObjectField("땔 시 애니메이션", releaseAnimation, typeof(AnimationClip)) as AnimationClip;
            }
            else
                clickAnimation = EditorGUILayout.ObjectField("클릭 시 애니메이션", clickAnimation, typeof(AnimationClip)) as AnimationClip;

            #region 개행
            BlankSpace();
            #endregion

            if (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                GUI.enabled = false;
            if (GUILayout.Button("Create Button"))
            {
                CreateButtonScript();

                while (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                {
                    // 에디터 업데이트
                    EditorApplication.update();

                    // 에셋 초기화 (최신으로)
                    AssetDatabase.Refresh();
                }

                GameObject obj = EditorExtention.CreateChildObject(position, itemName, typeof(BoxCollider), typeof(Button), typeof(EventTrigger));
                UnityEngineInternal.APIUpdaterRuntimeServices.AddComponent(obj, "Assets/Menu/UI/Scripts/Commons/Editor/UIWindowEditor.cs (208,17)", buttonScriptName);
                GameObject obj2 = EditorExtention.CreateChildObject(obj.transform, texturePathName, typeof(Image));
                GameObject obj3 = EditorExtention.CreateChildObject(obj.transform, textPathName, typeof(Text));

                if (sprite != null) { obj2.GetComponent<Image>().sprite = sprite; obj2.GetComponent<Image>().SetNativeSize(); obj.GetComponent<BoxCollider>().size = new Vector3(obj2.GetComponent<Image>().preferredWidth, obj2.GetComponent<Image>().preferredHeight, 0); }
                if (text != null) { obj3.GetComponent<Text>().font = text; obj3.GetComponent<Text>().text = textString; obj3.GetComponent<Text>().alignment = textAnchor; obj3.GetComponent<Text>().fontSize = textSize; obj3.GetComponent<Text>().color = new Color(textColor.r, textColor.g, textColor.b, 1.0f); }

                obj.GetComponent<Button>().transition = Selectable.Transition.None;

                if (pressAnimation != null || releaseAnimation != null || clickAnimation != null) EditorExtention.AddComponent(obj, typeof(Animation));
                if (pressAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonAnimation));
                    obj.GetComponent<Animation>().AddClip(pressAnimation, "PressAnimation");
                }
                if (releaseAnimation != null) obj.GetComponent<Animation>().AddClip(releaseAnimation, "ReleaseAnimation");
                if (clickAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonClickAnimation));
                    obj.GetComponent<Animation>().AddClip(clickAnimation, "ClickAnimation");
                }

                Selection.activeGameObject = obj;
                popupButtonCheck = obj;
            }
            if (GUI.enabled == false)
                GUI.enabled = true;
        }

        private void TextButton()
        {
            EditorGUILayout.HelpBox("폰트에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            text = EditorGUILayout.ObjectField("폰트 등록", text, typeof(Font)) as Font;

            if (textString.Equals(string.Empty)) textString = "Text";
            textString = EditorGUILayout.TextField("폰트 텍스트 등록", textString);
            if (textPathName.Equals(string.Empty)) textPathName = "Text";
            textPathName = EditorGUILayout.TextField("만들어질 자식의 이름", textPathName);
            if (textSize <= 0) textSize = 14;
            textSize = EditorGUILayout.IntField("폰트 사이즈 등록", textSize);
            textAnchor = (TextAnchor)EditorGUILayout.EnumPopup("폰트 정렬 기준 등록", textAnchor);
            textColor = EditorGUILayout.ColorField("폰트 색상 등록", textColor);

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("애니메이션에 대해서 설정하는 부분입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            animationType = GUILayout.Toolbar(animationType, new string[2] { "Press & Release", "Click" });

            #region 개행
            BlankSpace();
            #endregion

            if (animationType == 0)
            {
                pressAnimation = EditorGUILayout.ObjectField("누를 시 애니메이션", pressAnimation, typeof(AnimationClip)) as AnimationClip;
                releaseAnimation = EditorGUILayout.ObjectField("땔 시 애니메이션", releaseAnimation, typeof(AnimationClip)) as AnimationClip;
            }
            else
                clickAnimation = EditorGUILayout.ObjectField("클릭 시 애니메이션", clickAnimation, typeof(AnimationClip)) as AnimationClip;

            #region 개행
            BlankSpace();
            #endregion

            if (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                GUI.enabled = false;
            if (GUILayout.Button("Create Button"))
            {
                CreateButtonScript();

                while (EditorExtention.GetFileCheckInFile(buttonScriptCreatePath, buttonScriptName, "cs") == false)
                {
                    // 에디터 업데이트
                    EditorApplication.update();

                    // 에셋 초기화 (최신으로)
                    AssetDatabase.Refresh();
                }

                GameObject obj = EditorExtention.CreateChildObject(position, itemName, typeof(BoxCollider), typeof(Button), typeof(EventTrigger));
                UnityEngineInternal.APIUpdaterRuntimeServices.AddComponent(obj, "Assets/Menu/UI/Scripts/Commons/Editor/UIWindowEditor.cs (208,17)", buttonScriptName);
                GameObject obj3 = EditorExtention.CreateChildObject(obj.transform, textPathName, typeof(Text));

                if (text != null) { obj3.GetComponent<Text>().font = text; obj3.GetComponent<Text>().text = textString; obj3.GetComponent<Text>().alignment = textAnchor; obj3.GetComponent<Text>().fontSize = textSize; obj3.GetComponent<Text>().color = new Color(textColor.r, textColor.g, textColor.b, 1.0f); }

                obj.GetComponent<Button>().transition = Selectable.Transition.None;

                if (pressAnimation != null || releaseAnimation != null || clickAnimation != null) EditorExtention.AddComponent(obj, typeof(Animation));
                if (pressAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonAnimation));
                    obj.GetComponent<Animation>().AddClip(pressAnimation, "PressAnimation");
                }
                if (releaseAnimation != null) obj.GetComponent<Animation>().AddClip(releaseAnimation, "ReleaseAnimation");
                if (clickAnimation != null)
                {
                    EditorExtention.AddComponent(obj, typeof(ButtonClickAnimation));
                    obj.GetComponent<Animation>().AddClip(clickAnimation, "ClickAnimation");
                }

                Selection.activeGameObject = obj;
                popupButtonCheck = obj;
            }
            if (GUI.enabled == false)
                GUI.enabled = true;
        }

        private void CreateButtonScript()
        {
            System.IO.StreamWriter sw = new System.IO.StreamWriter(Application.dataPath + "/" + buttonScriptCreatePath + buttonScriptName + ".cs");
            
            string[] tempScriptData = tempScript;

            tempScriptData[3] = "namespace " + buttonScriptNamespaceName;
            tempScriptData[5] = tempScriptData[5].Replace("TempButtonScript", buttonScriptName);
            
            for (int i = 0; i < tempScript.Length; i++)
                sw.WriteLine(tempScriptData[i]);

            sw.Close();

            tempScript = tempScript2;

            // 에디터 업데이트
            EditorApplication.update();

            // 에셋 초기화 (최신으로)
            AssetDatabase.Refresh();

        }

        #endregion

        #region Popup Function...

        private void PopupSection()
        {
            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("공통적인 팝업의 내용을 만드는 곳 입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            if (popupName.Equals(string.Empty)) popupName = "TempPopup";
            popupName = EditorGUILayout.TextField("팝업 이름", popupName);
            popupPosition = EditorGUILayout.ObjectField("만들어질 경로", popupPosition, typeof(Transform)) as Transform;


            #region 개행
            BlankSpace();
            #endregion

            if (popupScriptName.Equals(string.Empty)) popupScriptName = "TempPopupScript";
            popupScriptName = EditorGUILayout.TextField("팝업 스크립트의 이름", popupScriptName);
            if (popupScriptNamespaceName.Equals(string.Empty)) popupScriptNamespaceName = "Menu.UI";
            popupScriptNamespaceName = EditorGUILayout.TextField("팝업 스크립트 네임스페이스", popupScriptNamespaceName);
            if (popupScriptCreatePath.Equals(string.Empty)) popupScriptCreatePath = "Menu/UI/Scripts/";
            popupScriptCreatePath = EditorGUILayout.TextField("팝업 스크립트 생성 위치", popupScriptCreatePath);

            if (GUILayout.Button("스크립트 생성"))
                CreatePopupScript();

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("팝업 뒷 배경의 내용을 만드는 곳 입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            if (popupBGName.Equals(string.Empty)) popupBGName = "Background";
            popupBGName = EditorGUILayout.TextField("뒷 배경 이름", popupBGName);
            popupBGColor = EditorGUILayout.ColorField("뒷 배경 색상", popupBGColor);
            popupStartBGAnimation = EditorGUILayout.ObjectField("뒷 배경 시작 애니메이션", popupStartBGAnimation, typeof(AnimationClip)) as AnimationClip;
            popupCloseBGAnimation = EditorGUILayout.ObjectField("뒷 배경 종료 애니메이션", popupCloseBGAnimation, typeof(AnimationClip)) as AnimationClip;

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("팝업 패널의 내용을 만드는 곳 입니다.", MessageType.Info);
            
            #region 개행
            BlankSpace();
            #endregion

            if (popupPanelName.Equals(string.Empty)) popupPanelName = "TempPanel";
            popupPanelName = EditorGUILayout.TextField("패널 이름", popupPanelName);
            popupPanelBGImg = EditorGUILayout.ObjectField("패널 뒷배경 이미지 등록", popupPanelBGImg, typeof(Sprite)) as Sprite;
            popupStartPanelAnimation = EditorGUILayout.ObjectField("패널 시작 애니메이션", popupStartPanelAnimation, typeof(AnimationClip)) as AnimationClip;
            popupClosePanelAnimation = EditorGUILayout.ObjectField("패널 종료 애니메이션", popupClosePanelAnimation, typeof(AnimationClip)) as AnimationClip;

            #region 개행
            BlankSpace();
            #endregion

            EditorGUILayout.HelpBox("팝업 패널에 닫기 버튼을 만드는 곳 입니다.", MessageType.Info);

            #region 개행
            BlankSpace();
            #endregion

            popupCloseButton = EditorGUILayout.ObjectField("닫기 버튼 오브젝트", popupCloseButton, typeof(ButtonUI)) as ButtonUI;

            #region 개행
            BlankSpace();
            #endregion

            if(popupCloseButton == null)
            {
                popupButtonFold = EditorGUILayout.BeginToggleGroup("버튼 생성", popupButtonFold);

                popupCloseButtonCreateScrollPos = EditorGUILayout.BeginScrollView(popupCloseButtonCreateScrollPos, GUILayout.Height(300.0f));

                ButtonSection();
                if (popupButtonCheck != null) { popupCloseButton = popupButtonCheck.GetComponent<ButtonUI>(); popupButtonCheck = null; }

                EditorGUILayout.EndScrollView();

                EditorGUILayout.EndToggleGroup();
            }


            #region 개행
            BlankSpace();
            #endregion

            if (EditorExtention.GetFileCheckInFile(popupScriptCreatePath, popupScriptName, "cs") == false)
                GUI.enabled = false;

            if (GUILayout.Button("팝업 생성"))
            {
                GameObject body = CreatePopupBody();
                UnityEngineInternal.APIUpdaterRuntimeServices.AddComponent(body, "Assets/Menu/UI/Scripts/Commons/Editor/UIWindowEditor.cs (864,17)", popupScriptName);

                GameObject bg = CreatePopupBackGround(body.GetComponent<Transform>());
                GameObject panel = CreatePopupPanel(body.GetComponent<Transform>());
                CreatePopupCloseButton(panel.GetComponent<Transform>());

                Selection.activeGameObject = body;
            }

        }

        private GameObject CreatePopupBody()
        {
            return EditorExtention.CreateChildObject(popupPosition, popupName);
        }

        private GameObject CreatePopupBackGround(Transform root)
        {
            GameObject obj = EditorExtention.CreateChildObject(root, popupBGName, typeof(Image), typeof(BoxCollider));

            RectTransform tr = obj.GetComponent<RectTransform>();
            tr.sizeDelta = new Vector2(EditorExtention.GetMainGameViewSize().x + 10, EditorExtention.GetMainGameViewSize().y + 10);
            obj.GetComponent<BoxCollider>().size = new Vector3(tr.sizeDelta.x, tr.sizeDelta.y, 0);
            Image img = obj.GetComponent<Image>();
            img.color = popupBGColor;

            if (popupStartBGAnimation != null || popupCloseBGAnimation != null)
            {
                obj.AddComponent<Animation>();

                Animation ani = obj.GetComponent<Animation>();
                if (popupStartBGAnimation != null)
                    ani.AddClip(popupStartBGAnimation, "StartAnimation");
                if (popupCloseBGAnimation != null)
                    ani.AddClip(popupCloseBGAnimation, "CloseAnimation");
            }

            return obj;
        }

        private GameObject CreatePopupPanel(Transform root)
        {
            GameObject obj = EditorExtention.CreateChildObject(root, popupPanelName, typeof(Image));
            if (popupPanelBGImg != null) { obj.GetComponent<Image>().sprite = popupPanelBGImg; obj.GetComponent<Image>().SetNativeSize(); }
            
            if (popupStartPanelAnimation != null || popupClosePanelAnimation != null)
            {
                obj.AddComponent<Animation>();

                Animation ani = obj.GetComponent<Animation>();
                if (popupStartPanelAnimation != null)
                    ani.AddClip(popupStartPanelAnimation, "StartAnimation");
                if (popupClosePanelAnimation != null)
                    ani.AddClip(popupClosePanelAnimation, "CloseAnimation");
            }

            return obj;
        }

        private void CreatePopupCloseButton(Transform panel)
        {
            if(popupCloseButton != null)
                popupCloseButton.transform.SetParent(panel, false);
        }
        
        private void CreatePopupScript()
        {
            System.IO.StreamWriter sw = new System.IO.StreamWriter(Application.dataPath + "/" + popupScriptCreatePath + popupScriptName + ".cs");

            string[] tempScriptData = tempPopupScript;

            tempScriptData[2] = "namespace " + buttonScriptNamespaceName;
            tempScriptData[4] = tempScriptData[4].Replace("TempPopupScript", popupScriptName);
             
            for (int i = 0; i < tempPopupScript.Length; i++)
                sw.WriteLine(tempScriptData[i]);

            sw.Close();

            tempPopupScript = tempPopupScript2;

            // 에디터 업데이트
            EditorApplication.update();

            // 에셋 초기화 (최신으로)
            AssetDatabase.Refresh();
        }

        #endregion

        #region Scroll Functions...

        

        #endregion
        
        [MenuItem("GameObject/Editor Utilities/Show UI Window", false, -100)]
        public static void ShowWindow()
        {
            window = GetWindow(typeof(UIWindowEditor), false, "UI Editor Window", true);
            window.minSize = new Vector2(800, 600);
            window.maxSize = new Vector2(800, 600);
        }
    }

}

